/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with {library}; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package ch.hortis.sonar.model;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;

import javax.persistence.*;
import java.io.Serializable;
import java.util.Date;

@Entity
@Table(name = "snapshots")
@NamedQueries({@NamedQuery(name = Snapshot.SQL_SELECT_ALL, query = "SELECT m FROM Snapshot m, MavenProject pr"
    + " WHERE m.mavenProject = pr.id" + " AND pr.id = :idProject ORDER BY m.createdAt"),
@NamedQuery(name = Snapshot.SQL_SELECT_FOR_DAY, query = "SELECT s FROM Snapshot s, MavenProject pr"
    + " WHERE s.mavenProject = pr.id" + " AND pr.id = :idProject AND s.last=false and s.status=:status"
    + " AND day(s.createdAt) = :day AND month(s.createdAt) = :month AND year(s.createdAt) = :year ORDER BY s.createdAt ASC"),
@NamedQuery(name = Snapshot.SQL_SELECT_TO_PURGE, query = "SELECT s FROM Snapshot s"
    + " WHERE s.purged=false AND s.last=false AND s.status='" + Snapshot.STATUS_PROCESSED + "' AND s.root IS NULL"),
@NamedQuery(name = Snapshot.SQL_SELECT_UNPROCESSED_FOR_DATE, query = "SELECT s FROM Snapshot s"
    + " WHERE s.createdAt < :createdAt AND s.status<>'" + Snapshot.STATUS_PROCESSED + "' AND s.parent is NULL")
    })
public class Snapshot implements Serializable {

  public final static String SQL_SELECT_ALL = "Snapshot.selectAll";
  public final static String SQL_SELECT_FOR_DAY = "Snapshot.selectForDay";
  public static final String SQL_SELECT_TO_PURGE = "Snapshot.selectSnapshotsToPurge";
  public static final String SQL_SELECT_UNPROCESSED_FOR_DATE = "Snapshot.selectUnprocessedFordate";

  private static final long serialVersionUID = -6637859483638004953L;

  public final static String STATUS_UNPROCESSED = "U";
  public final static String STATUS_TO_PROCESS = "T";
  public final static String STATUS_PROCESSING = "W";
  public final static String STATUS_CONSOLIDATED = "C";
  public final static String STATUS_PROCESSED = "P";

  @Id
  @Column(name = "id")
  @SequenceGenerator(name = "SNAPSHOTS_SEQ", sequenceName = "SNAPSHOTS_SEQ")
  @GeneratedValue(strategy = GenerationType.AUTO, generator = "SNAPSHOTS_SEQ")
  private Integer id;

  @ManyToOne(fetch = FetchType.LAZY)
  @JoinColumn(name = "project_id", updatable = false, nullable = false)
  @Cache(usage = CacheConcurrencyStrategy.READ_WRITE)
  private MavenProject mavenProject;

  @Temporal(TemporalType.TIMESTAMP)
  @Column(name = "created_at", updatable = false, nullable = false)
  private Date createdAt;

  @Column(name = "version", updatable = true, nullable = true, length = 32)
  private String version;

  @Column(name = "islast")
  private Boolean last = Boolean.FALSE;

  @Column(name = "status")
  private String status = STATUS_UNPROCESSED;

  @Column(name = "purged")
  private Boolean purged = Boolean.FALSE;

  @Column(name = "scope", updatable = false, nullable = true, length = 3)
  private String scope;

  @Column(name = "qualifier", updatable = false, nullable = true, length = 3)
  private String qualifier;

  @ManyToOne(fetch = FetchType.LAZY, cascade = {CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH})
  @JoinColumn(name = "root_snapshot_id", updatable = true, nullable = true)
  @Cache(usage = CacheConcurrencyStrategy.READ_WRITE)
  private Snapshot root;

  @ManyToOne(fetch = FetchType.LAZY, cascade = {CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH})
  @JoinColumn(name = "parent_snapshot_id", updatable = true, nullable = true)
  @Cache(usage = CacheConcurrencyStrategy.READ_WRITE)
  private Snapshot parent;

  public Snapshot() {
  }

  public Snapshot(MavenProject entity, boolean last, String status, Date date) {
    this();
    setMavenProject(entity);
    this.status = status;
    this.last = last;
    this.createdAt = date;
  }

  public Date getCreatedAt() {
    return createdAt;
  }

  public void setCreatedAt(Date createdAt) {
    this.createdAt = createdAt;
  }

  public Integer getId() {
    return id;
  }

  public void setId(Integer id) {
    this.id = id;
  }

  public MavenProject getMavenProject() {
    return mavenProject;
  }

  public void setMavenProject(MavenProject mavenProject) {
    this.mavenProject = mavenProject;
    this.scope = mavenProject.getScope();
    this.qualifier = mavenProject.getQualifier();
  }

  public String getVersion() {
    return version;
  }

  public void setVersion(String version) {
    this.version = version;
  }

  public Snapshot getParent() {
    return parent;
  }

  public void setParent(Snapshot parent) {
    this.parent = parent;
  }

  public Boolean getLast() {
    return last;
  }

  public void setLast(Boolean last) {
    this.last = last;
  }

  public String getStatus() {
    return status;
  }

  public void setStatus(String status) {
    this.status = status;
  }

  public Boolean getPurged() {
    return purged;
  }

  public void setPurged(Boolean purged) {
    this.purged = purged;
  }

  public String getScope() {
    return scope;
  }

  public void setScope(String scope) {
    this.scope = scope;
  }

  public String getQualifier() {
    return qualifier;
  }

  public void setQualifier(String qualifier) {
    this.qualifier = qualifier;
  }

  public Snapshot getRoot() {
    return root;
  }

  public void setRoot(Snapshot root) {
    this.root = root;
  }

  @Override
  public boolean equals(Object obj) {
    if (!(obj instanceof Snapshot)) {
      return false;
    }
    if (this == obj) {
      return true;
    }
    Snapshot other = (Snapshot) obj;
    return new EqualsBuilder()
        .append(mavenProject, other.getMavenProject())
        .append(createdAt, other.getCreatedAt())
        .isEquals();
  }

  @Override
  public int hashCode() {
    return new HashCodeBuilder(17, 37)
        .append(mavenProject)
        .append(createdAt)
        .toHashCode();
  }
}
